#ifndef INCLUDED_IMODULECONSTRUCTIONSMARTRECEIVER_H
#define INCLUDED_IMODULECONSTRUCTIONSMARTRECEIVER_H
#include <string>

namespace utils {
	class Buffer;
}

 /**
  * TODO: descr. of the interface
  */

class IModuleConstructionSmartReceiver
{
public:
  /**
   * Fgt dem Laufzeitsystem ein neues Modul des Typs moduleClassID hinzu.
   * @param moduleClassID Bestimmt die Art des Moduls das hinzugefgt wird.
   * @param modID Eine eindeutige ID fr das neue Modul.
   */
	virtual void moduleAdded( const std::string& graphName, const std::string& moduleClassName, int modID ) = 0;

  /**
   * Verknpft den Eingang eines Moduls mit dem Ausgang eines anderen.
   * @param moduleID1 Die ID des Quellmoduls
   * @param outputIndex Der Index des Ausgangs des Quellmoduls
   * @param moduleID2 Die ID des Zielmoduls
   * @param inputIndex Der Index des eingangs des Zielmoduls
   * @exception runtime_error Wird geworfen wenn eine moduleID
   *            ungltig ist, die out/inputIndex ungltig ist oder der
   *            Typ des Ausgangs nicht mit dem des Eingangs bereinstimmt.
   */
	virtual void modulesConnected( const std::string& graphName, int moduleID1, int outputIndex, int moduleID2, int inputIndex ) = 0;

  /**
   * Lst die Verknpfung eines Eingangs von dem zu moduleID gehrigen Modul.
   * @param moduleID Das Modul von dem ein Eingang gelscht werden soll
   * @param inputIndex Der index des Eingangs
   * @exception runtime_error Wird geworfen wenn moduleID oder inputIndex
   *       ungltig ist.
   */
	virtual void modulesDisconnected( const std::string& graphName, int moduleID, int inputIndex ) = 0;

  /**
   * Lscht ein Modul.
   * @param moduleID Das Modul das gelscht wird
   */
	virtual void moduleDeleted( const std::string& graphName, int moduleID ) = 0;

  /**
   *
   */
	virtual void newGraphCreated( const std::string& graphName ) = 0;

  /**
   *
   */
	virtual void graphChanged( const std::string& graphName ) = 0;

  /**
   *
   */
	virtual void graphDeleted( const std::string& graphName ) = 0;

};
#endif
